@@ -1,34 +1,33 @@
 from datetime import datetime, timedelta
-from typing import List, Optional
-from pytz import timezone
+from typing import Dict, List, Set, Optional, Callable
+
+from pyTSL import Client, DoubleToDatetime
 
 from vnpy.trader.setting import SETTINGS
 from vnpy.trader.constant import Exchange, Interval
 from vnpy.trader.object import BarData, TickData, HistoryRequest
-from vnpy.trader.utility import extract_vt_symbol
+from vnpy.trader.utility import extract_vt_symbol, ZoneInfo
 from vnpy.trader.datafeed import BaseDatafeed
 
-from .pyTSL import Client, DoubleToDatetime
 
-
-CHINA_TZ = timezone("Asia/Shanghai")
-
-EXCHANGE_MAP = {
+EXCHANGE_MAP: Dict[Exchange, str] = {
     Exchange.SSE: "SH",
     Exchange.SZSE: "SZ"
 }
 
-INTERVAL_MAP = {
+INTERVAL_MAP: Dict[Interval, str] = {
     Interval.MINUTE: "cy_1m",
     Interval.HOUR: "cy_60m",
     Interval.DAILY: "cy_day",
 }
 
-SHIFT_MAP = {
+SHIFT_MAP: Dict[Interval, timedelta] = {
     Interval.MINUTE: timedelta(minutes=1),
     Interval.HOUR: timedelta(hours=1),
 }
 
+CHINA_TZ = ZoneInfo("Asia/Shanghai")
+
 
 class TinysoftDatafeed(BaseDatafeed):
     """天软数据服务接口"""
@@ -41,7 +40,7 @@ class TinysoftDatafeed(BaseDatafeed):
         self.client: Client = None
         self.inited: bool = False
 
-    def init(self) -> bool:
+    def init(self, output: Callable = print) -> bool:
         """初始化"""
         if self.inited:
             return True
@@ -53,28 +52,31 @@ class TinysoftDatafeed(BaseDatafeed):
             443
         )
 
-        n = self.client.login()
+        n: int = self.client.login()
         if n != 1:
+            output("天软数据服务初始化失败：用户名密码错误！")
             return False
 
         self.inited = True
         return True
 
-    def query_bar_history(self, req: HistoryRequest) -> Optional[List[BarData]]:
+    def query_bar_history(self, req: HistoryRequest, output: Callable = print) -> Optional[List[BarData]]:
         """查询K线数据"""
         if not self.inited:
-            self.init()
+            n: bool = self.init(output)
+            if not n:
+                return []
 
         symbol, exchange = extract_vt_symbol(req.vt_symbol)
-        tsl_exchange = EXCHANGE_MAP.get(exchange, "")
-        tsl_interval = INTERVAL_MAP[req.interval]
+        tsl_exchange: str = EXCHANGE_MAP.get(exchange, "")
+        tsl_interval: str = INTERVAL_MAP[req.interval]
 
         bars: List[BarData] = []
 
-        start_str = req.start.strftime("%Y%m%d")
-        end_str = req.end.strftime("%Y%m%d")
+        start_str: str = req.start.strftime("%Y%m%d")
+        end_str: str = req.end.strftime("%Y%m%d")
 
-        cmd = (
+        cmd: str = (
             f"setsysparam(pn_cycle(),{tsl_interval}());"
             "return select * from markettable "
             f"datekey {start_str}T to {end_str}T "
@@ -84,17 +86,17 @@ class TinysoftDatafeed(BaseDatafeed):
 
         if not result.error():
             data = result.value()
-            shift = SHIFT_MAP.get(req.interval, None)
+            shift: timedelta = SHIFT_MAP.get(req.interval, None)
 
             for d in data:
                 dt: datetime = DoubleToDatetime(d["date"])
                 if shift:
                     dt -= shift
 
-                bar = BarData(
+                bar: BarData = BarData(
                     symbol=symbol,
                     exchange=exchange,
-                    datetime=CHINA_TZ.localize(dt),
+                    datetime=dt.replace(tzinfo=CHINA_TZ),
                     interval=req.interval,
                     open_price=d["open"],
                     high_price=d["high"],
@@ -113,29 +115,37 @@ class TinysoftDatafeed(BaseDatafeed):
 
         return bars
 
-    def query_tick_history(self, req: HistoryRequest) -> Optional[List[TickData]]:
+    def query_tick_history(self, req: HistoryRequest, output: Callable = print) -> Optional[List[TickData]]:
         """查询Tick数据"""
         if not self.inited:
-            self.init()
+            n: bool = self.init(output)
+            if not n:
+                return []
 
         symbol, exchange = extract_vt_symbol(req.vt_symbol)
-        tsl_exchange = EXCHANGE_MAP.get(exchange, "")
+        tsl_exchange: str = EXCHANGE_MAP.get(exchange, "")
 
         ticks: List[TickData] = []
+        dts: Set[datetime] = set()
 
         dt: datetime = req.start
         while dt <= req.end:
-            date_str = dt.strftime("%Y%m%d")
-            cmd = f"return select * from tradetable datekey {date_str}T to {date_str}T+16/24 of '{tsl_exchange}{symbol}' end ; "
+            date_str: str = dt.strftime("%Y%m%d")
+            cmd: str = f"return select * from tradetable datekey {date_str}T to {date_str}T+16/24 of '{tsl_exchange}{symbol}' end ; "
             result = self.client.exec(cmd)
 
             if not result.error():
                 data = result.value()
                 for d in data:
-                    dt = DoubleToDatetime(d["date"])
-                    dt = CHINA_TZ.localize(dt)
+                    dt: datetime = DoubleToDatetime(d["date"])
+                    dt: datetime = dt.replace(tzinfo=CHINA_TZ)
 
-                    tick = TickData(
+                    # 解决期货缺失毫秒时间戳的问题
+                    if dt in dts:
+                        dt = dt.replace(microsecond=500000)
+                    dts.add(dt)
+
+                    tick: TickData = TickData(
                         symbol=symbol,
                         exchange=exchange,
                         name=d["StockName"],
@@ -174,6 +184,14 @@ class TinysoftDatafeed(BaseDatafeed):
                     if not tsl_exchange:
                         tick.open_interest = d["sectional_cjbs"]
 
+                    # 基金获取IOPV字段
+                    if d["syl2"]:
+                        iopv: float = d["syl2"]
+                        if exchange == Exchange.SZSE:       # 深交所的IOPV要除以100才是每股
+                            iopv /= 100
+
+                        tick.extra = {"iopv": iopv}
+
                     ticks.append(tick)
 
             dt += timedelta(days=1)
